package cd.casic.ci.process.engine.worker.afl;

import cd.casic.ci.process.common.WorkAtom;
import cd.casic.ci.process.dto.req.resource.ResourceQueryReq;
import cd.casic.ci.process.dto.resp.taskResource.TaskResourceFindResp;
import cd.casic.ci.process.engine.constant.AFLConstant;
import cd.casic.ci.process.engine.constant.DIYImageExecuteCommandConstant;
import cd.casic.ci.process.engine.runContext.TaskRunContext;
import cd.casic.ci.process.engine.worker.base.DockerWorker;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.util.Map;

@WorkAtom(taskType = "AFL")
@Slf4j
public class AFLWorker extends DockerWorker {


    @Override
    public void execute(TaskRunContext context) {
        int statusCode = -1;
        Map<String, Object> localVariables = context.getLocalVariables();
        if (context.getContextDef() instanceof PipTask taskDef) {
            log.info(taskDef.getTaskName());
            Map<String, Object> taskProperties = taskDef.getTaskProperties();
            //从taskProperties中获取资源id
            String resourceType = taskProperties.get("resourceType").toString();
            String resourceId = taskProperties.get("resourceId").toString();
            // 待测试文件路径
            String binaryPath = taskProperties.get(AFLConstant.BINARY) instanceof String ? ((String) taskProperties.get(AFLConstant.BINARY)) : null;
            String output = taskProperties.get(AFLConstant.OUTPUT) instanceof String ? ((String) taskProperties.get(AFLConstant.OUTPUT)) : null;
            String input = taskProperties.get(AFLConstant.INPUT) instanceof String ? ((String) taskProperties.get(AFLConstant.INPUT)) : null;
            String workDir = taskProperties.get(AFLConstant.WORK_DIR) instanceof String ? ((String) taskProperties.get(AFLConstant.WORK_DIR)) : null;

            if (StringUtils.isEmpty(binaryPath) ||
                    StringUtils.isEmpty(output) ||
                    StringUtils.isEmpty(input) ||
                    StringUtils.isEmpty(workDir) ||
                    StringUtils.isEmpty(resourceId) ||
                    StringUtils.isEmpty(resourceType)) {
//                缺少参数
                toBadEnding();
            }
            ResourceQueryReq req = new ResourceQueryReq();
            req.setId(resourceId);
            req.setType(resourceType);
            TaskResourceFindResp resourceListByType = getResourceManagerService().findResourceListByType(req);
            if (CollectionUtils.isEmpty(resourceListByType.getDockerEndpointList())) {
                append(context,"当前机器不支持docker");
                return;
            }
            String commandScript = "docker run -v "+workDir+":/test  -it aflplusplus/aflplusplus bash\n" +
                    "cd /test\n" +
                    "afl-fuzz -i "+input+" -o "+output+" -t 3000 -Q "+binaryPath+" @@";
            try {
                //将节点的配置信息反编译成对象
                log.info("构建脚本" + commandScript);
                // 获取docker 暂时先写固定值
                dockerRun(commandScript,resourceListByType.getDockerEndpointList().get(0),context);
            } catch (Exception e) {
                String errorMessage = "该节点配置信息为空，请先配置该节点信息" + "\r\n";
                log.error("执行ssh失败:", e);
                append(context, errorMessage);
                toBadEnding();
            }
            localVariables.put(DIYImageExecuteCommandConstant.STATUS_CODE, statusCode);
        }
    }
}
