package cd.casic.ci.process.engine.worker.afl;


import cd.casic.ci.process.common.WorkAtom;
import cd.casic.ci.process.dto.req.resource.ResourceQueryReq;
import cd.casic.ci.process.dto.resp.taskResource.TaskResourceFindResp;
import cd.casic.ci.process.engine.constant.DIYImageExecuteCommandConstant;
import cd.casic.ci.process.engine.constant.TestCaseGenerationConstant;
import cd.casic.ci.process.engine.runContext.TaskRunContext;
import cd.casic.ci.process.engine.worker.base.SshWorker;
import cd.casic.ci.process.process.dataObject.pipeline.PipPipeline;
import cd.casic.ci.process.process.dataObject.resource.PipResourceMachine;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.ci.process.util.CryptogramUtil;
import cd.casic.ci.process.util.SftpUploadUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.util.Map;

@WorkAtom(taskType = "TEST_CASE_GENERATION")
@Slf4j
public class TestCaseGenerationWorker extends SshWorker {
    @Override
    public void execute(TaskRunContext context) {
        int statusCode = -1;
        Map<String, Object> localVariables = context.getLocalVariables();
        if (context.getContextDef() instanceof PipTask taskDef) {
            log.info(taskDef.getTaskName());
            Map<String, Object> taskProperties = taskDef.getTaskProperties();
            Object caseType = taskProperties.get(TestCaseGenerationConstant.CASE_TYPE_KEY);
            String resourceType = taskProperties.get("resourceType").toString();
            String resourceId = taskProperties.get("resourceId").toString();
            if (StringUtils.isEmpty(resourceId) || StringUtils.isEmpty(resourceType)) {
//                    缺少参数
                toBadEnding();
            }
            ResourceQueryReq req = new ResourceQueryReq();
            req.setId(resourceId);
            req.setType(resourceType);
            TaskResourceFindResp resourceListByType = getResourceManagerService().findResourceListByType(req);
            if (CollectionUtils.isEmpty(resourceListByType.getResourceMachineList())) {
                append(context,"当前机器不支持machine");
                return;
            }
            //如果machineId为0，则说明该节点没有配置机器，则使用开始节点的机器
            PipResourceMachine resourceMachine = resourceListByType.getResourceMachineList().get(0);
            if (TestCaseGenerationConstant.CASE_TYPE_AI.equals(caseType)) {
                Object commandScriptObj = taskProperties.get(TestCaseGenerationConstant.COMMAND_SCRIPT);
                String commandScript = commandScriptObj instanceof String ? ((String) commandScriptObj) : null;
                String seedTarget = taskProperties.get(TestCaseGenerationConstant.SEED_TARGET) instanceof String ? ((String) taskProperties.get(TestCaseGenerationConstant.SEED_TARGET)) : null;
                if (StringUtils.isEmpty(commandScript)) {
//                缺少参数
                    toBadEnding();
                }
                if (StringUtils.isNotEmpty(seedTarget)) {
                    commandScript += " --output-dir " + seedTarget + " --count 100";
                } else {
                    commandScript += " --output-dir case --count 100";
                }
                try {
                    //将节点的配置信息反编译成对象
                    log.info("构建脚本" + commandScript);
                    //获取机器
                    statusCode = shell(resourceMachine, null, context,
                            "echo \"测试用例生成\"",
                            "cd /home/casic/706/yunqi",// 基础目录
                            commandScript
                    );
                } catch (Exception e) {
                    String errorMessage = "该节点配置信息为空，请先配置该节点信息" + "\r\n";
                    log.error("执行ssh失败:", e);
                    append(context, errorMessage);
                    toBadEnding();
                }
                if (statusCode == 0) {
                    log.info("节点执行完成");
                } else {
                    log.error("节点执行失败");
                }
                localVariables.put(DIYImageExecuteCommandConstant.STATUS_CODE, statusCode);
            } else {
                // 文件上传
                String filePath = taskProperties.get(TestCaseGenerationConstant.SEED_SOURCE) instanceof String ? ((String) taskProperties.get(TestCaseGenerationConstant.SEED_SOURCE)) : null;
                if (StringUtils.isEmpty(filePath)) {
                    append(context,"缺少文件");
                    toBadEnding();
                }
                File file = new File(filePath);
                String seedTarget = taskProperties.get(TestCaseGenerationConstant.SEED_TARGET) instanceof String ? ((String) taskProperties.get(TestCaseGenerationConstant.SEED_TARGET)) : null;
                String basePath = "/home/casic/706/yunqi/";
                            if (seedTarget.startsWith("/")) {
                    seedTarget=seedTarget.substring(1);
                }
                seedTarget = basePath+seedTarget;
                // 将文件上传到服务器的 目录底下
                            try {
                    SftpUploadUtil.uploadFileViaSftp(
                            resourceMachine.getMachineHost()
                            ,Integer.valueOf(resourceMachine.getSshPort()),resourceMachine.getUsername(),resourceMachine.getPassword(),"",filePath,seedTarget,file.getName());
                } catch (SftpUploadUtil.SftpUploadException e) {
                    append(context,"seed文件上传失败");
                    log.error("seed文件上传失败",e);
                }
            }
        }
    }
}
